@@ -7,7 +7,7 @@ import tornado.web
 from wpull.backport.testing import unittest
 from wpull.errors import ConnectionRefused, SSLVerficationError
 from wpull.http import (Request, Connection, NetworkError, ProtocolError, Client,
-    ConnectionPool, parse_charset, Response, RedirectTracker, HostConnectionPool)
+    ConnectionPool, parse_charset, Response, HostConnectionPool)
 from wpull.recorder import DebugPrintRecorder
 from wpull.testing.badapp import BadAppTestCase
 
@@ -79,6 +79,7 @@ class TestConnection(BadAppTestCase):
         response = yield self.fetch('/')
         self.assertEqual(200, response.status_code)
         self.assertEqual(b'hello world!', response.body.content)
+        self.assertTrue(response.url_info)
 
     @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
     def test_basic_content_length(self):
@@ -119,7 +120,7 @@ class TestConnection(BadAppTestCase):
         request = Request.new(self.get_url('/buffer_overflow'))
         try:
             yield connection.fetch(request)
-        except ProtocolError:
+        except (ProtocolError, NetworkError):
             pass
         else:
             self.fail()
@@ -392,51 +393,6 @@ class TestHTTP(unittest.TestCase):
         self.assertEqual(404, code)
         self.assertEqual(b'N\x99t \x0eounz'.decode('latin-1'), msg)
 
-    def test_redirect_tracker(self):
-        tracker = RedirectTracker(5)
-
-        self.assertFalse(tracker.is_redirect())
-        self.assertFalse(tracker.is_repeat())
-        self.assertFalse(tracker.exceeded())
-        self.assertFalse(tracker.next_location())
-        self.assertEqual(0, tracker.count())
-
-        response = Response('HTTP/1.1', 200, '')
-
-        tracker.load(response)
-
-        self.assertFalse(tracker.is_redirect())
-        self.assertFalse(tracker.is_repeat())
-        self.assertFalse(tracker.exceeded())
-        self.assertFalse(tracker.next_location())
-        self.assertEqual(0, tracker.count())
-
-        response = Response('HTTP/1.1', 303, '')
-        response.fields['location'] = '/test'
-
-        tracker.load(response)
-
-        self.assertTrue(tracker.is_redirect())
-        self.assertFalse(tracker.is_repeat())
-        self.assertFalse(tracker.exceeded())
-        self.assertEqual('/test', tracker.next_location())
-        self.assertEqual(1, tracker.count())
-
-        response = Response('HTTP/1.1', 307, '')
-        response.fields['location'] = '/test'
-
-        tracker.load(response)
-        tracker.load(response)
-        tracker.load(response)
-        tracker.load(response)
-        tracker.load(response)
-
-        self.assertTrue(tracker.is_redirect())
-        self.assertTrue(tracker.is_repeat())
-        self.assertTrue(tracker.exceeded())
-        self.assertEqual('/test', tracker.next_location())
-        self.assertEqual(6, tracker.count())
-
 
 class TestSSL(tornado.testing.AsyncHTTPSTestCase):
     def get_app(self):
